"""Import an existing :term:`LAN` Switch Interconnect into the subscription database."""

from uuid import uuid4

from orchestrator import step, workflow
from orchestrator.forms import FormPage
from orchestrator.targets import Target
from orchestrator.types import FormGenerator, State, SubscriptionLifecycle
from orchestrator.workflow import StepList, begin, done
from orchestrator.workflows.steps import resync, set_status, store_process_subscription
from products.product_types.lan_switch_interconnect import ImportedLanSwitchInterconnectInactive

from gso.cli.imports import LanSwitchInterconnectRouterSideImportModel, LanSwitchInterconnectSwitchSideImportModel
from gso.products import ProductName
from gso.products.product_blocks.lan_switch_interconnect import (
    LanSwitchInterconnectRouterSideBlockInactive,
    LanSwitchInterconnectSwitchSideBlockInactive,
)
from gso.services.partners import get_partner_by_name
from gso.services.subscriptions import get_product_id_by_name
from gso.utils.types.ip_address import AddressSpace, IPv4NetworkType


def _initial_input_form_generator() -> FormGenerator:
    class ImportLanSwitchInterconnect(FormPage):
        lan_switch_interconnect_description: str
        lan_switch_interconnect_ip_network: IPv4NetworkType | None
        address_space: AddressSpace
        minimum_links: int
        router_side: LanSwitchInterconnectRouterSideImportModel
        switch_side: LanSwitchInterconnectSwitchSideImportModel

    user_input = yield ImportLanSwitchInterconnect
    return user_input.model_dump() | {"partner": "GEANT"}


@step("Create subscription")
def create_subscription(partner: str) -> State:
    """Create a new subscription object."""
    partner_id = get_partner_by_name(partner)["partner_id"]
    product_id = get_product_id_by_name(ProductName.IMPORTED_LAN_SWITCH_INTERCONNECT)
    subscription = ImportedLanSwitchInterconnectInactive.from_product_id(product_id, partner_id)

    return {"subscription": subscription, "subscription_id": subscription.subscription_id}


@step("Initialize subscription")
def initialize_subscription(
    subscription: ImportedLanSwitchInterconnectInactive,
    lan_switch_interconnect_description: str,
    lan_switch_interconnect_ip_network: IPv4NetworkType | None,
    address_space: AddressSpace,
    minimum_links: int,
    lan_switch_interconnect_router_side: dict,
    lan_switch_interconnect_switch_side: dict,
) -> State:
    """Initialize the subscription using input data."""
    subscription.lan_switch_interconnect.lan_switch_interconnect_description = lan_switch_interconnect_description
    subscription.lan_switch_interconnect.lan_switch_interconnect_ip_network = lan_switch_interconnect_ip_network
    subscription.lan_switch_interconnect.address_space = address_space
    subscription.lan_switch_interconnect.minimum_links = minimum_links
    subscription.lan_switch_interconnect.router_side = LanSwitchInterconnectRouterSideBlockInactive.new(
        uuid4(), **lan_switch_interconnect_router_side
    )
    subscription.lan_switch_interconnect.switch_side = LanSwitchInterconnectSwitchSideBlockInactive.new(
        uuid4(), **lan_switch_interconnect_switch_side
    )

    return {"subscription": subscription}


@workflow(
    "Create Imported LAN Switch Interconnect", initial_input_form=_initial_input_form_generator, target=Target.CREATE
)
def create_imported_lan_switch_interconnect() -> StepList:
    """Create an imported :term:`LAN` Switch Interconnect without provisioning it."""
    return (
        begin
        >> create_subscription
        >> store_process_subscription(Target.CREATE)
        >> initialize_subscription
        >> set_status(SubscriptionLifecycle.ACTIVE)
        >> resync
        >> done
    )
